-- Miscellaneous helper functions
--[[
Name: YakHelpers-2.0
Revision: 0
Author: Vic Putz (vputz@nyx.net)
Description: Some handy helper functions, wrapped in a library
Dependencies: AceLibrary
]]

local MAJOR_VERSION = "YakHelpers-2.0"
local MINOR_VERSION = 0

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local YakHelpers = {}

function YakHelpers:print( msg )
   DEFAULT_CHAT_FRAME:AddMessage( tostring(msg) )
end

function YakHelpers:print_table( t )
   -- very juvenile print-each-item implementation
   for k,v in pairs(t) do
      self:print(k .. "->" ..tostring(v))
   end
end

function YakHelpers:string_split(s)   
   local result={}
   for match in string.gmatch(s, "[^%s]+" ) do
      table.insert(result,match)
   end
   return result
end

function YakHelpers:console_command_string(msg)
   -- returns the "second command" and a table of arguments.  Does not
   -- handle quoted strings or spaces within an argument (spaces separate
   -- arguments).  Ie YakHelpers:console_command_string("/mymod command arg1 arg2")
   -- would return "command" as its first return value and {"arg1","arg2"} as
   -- its second return value.
   local commandlist = self:string_split(msg)
   local command = commandlist[1]
   table.remove(commandlist, 1)
   return command, commandlist
end

YakHelpers.YAK_LOGGER = {}

function YakHelpers.YAK_LOGGER:Create( log_status, text_color )
   -- text color is a triplet table, ie {1,0,0} for red, etc
   local log_status = log_status or false
   local text_color = text_color or { 1,1,1 }
   local result = { log=log_status, chatframe=DEFAULT_CHAT_FRAME, color=text_color }
   setmetatable( result, self )
   self.__index = self
   return result
end

function YakHelpers.YAK_LOGGER:SetLog( log_status )
   self.log = log_status
end

function YakHelpers.YAK_LOGGER:Log(msg)
   if self.log then
      self.chatframe:AddMessage(msg, self.color[1], self.color[2], self.color[3] )
   end
end

function YakHelpers:PlayerIsSolo()
   return (GetNumPartyMembers() == 0) and not self:PlayerIsInRaid()
end

function YakHelpers:PlayerIsInGroup()
   -- returns FALSE if player is in a raid!
   return ((not self:PlayerIsInRaid()) and (GetNumPartyMembers() > 0))
end

function YakHelpers:PlayerIsInRaid()
   return (GetNumRaidMembers() > 0)
end

function YakHelpers:BuffsMatchPattern(unit,p)
   --thanks for hint from Gello of Underhill for this idea
   local f,b 
   for i=1,24 do 
      b=UnitBuff(unit,i) or "" 
      f=f or strfind(b,p)
   end 
   if f then
      return true
   end
end

function YakHelpers:DebuffsMatchPattern(unit,p)
   --thanks for hint from Gello of Underhill for this idea
   local f,b 
   for i=1,16 do 
      b=UnitDebuff(unit,i) or "" 
      f=f or strfind(b,p)
   end 
   if f then
      return true
   end
end

function YakHelpers:NamedUnitIsInRaid(name, checkpets)
   for k,v in self:eachRaidID() do
      if UnitName(v) == name then
	 return true
      end
   end
   if checkpets then
      for k,v in self:eachRaidPetID() do
	 if UnitName(v) == name then
	    return true
	 end
      end
   end
   return false
end

function YakHelpers:IdOfUnitInRaid( name )
   for k,v in self:eachRaidID() do
      if UnitName(v) == name then
	 return v
      end
   end
   return nil
end

function YakHelpers:NamedUnitIsInParty(name, checkpets)
   for k,v in self:eachPartyID() do
      if UnitName(v) == name then
	 return true
      end
   end
   if checkpets then
      for k,v in self:eachPartyPetID() do
	 if UnitName(v) == name then
	    return true
	 end
      end
   end
   return false
end

function YakHelpers:IdOfUnitInParty( name )
   for k,v in self:eachPartyID() do
      if UnitName(v) == name then
	 return v
      end
   end
   return nil
end
YakHelpers.PARTYIDS = {
   "party1",
   "party2",
   "party3",
   "party4",
   "party5"
}

YakHelpers.PARTYPETIDS = {
   "partypet1",
   "partypet2",
   "partypet3",
   "partypet4",
   "partypet5"
}

function YakHelpers:partyiditer(i)
   i = i+1
   if (i < 5) and GetPartyMember( i ) then
      return  i,self.PARTYIDS[i]
   end
end

function YakHelpers:eachPartyID()
    return self.partyiditer, self, 0
end

function YakHelpers:partypetiditer( i )
   i = i+1
   if (i < 5) and GetPartyMember( i ) then
      return  i,self.PARTYIDS[i]
   end
end

function YakHelpers:eachPartyPetID()
    return self.partyiditer, self, 0
end

function  YakHelpers:partynameiter(i)
   i = i+1
   if (i < 5) and GetPartyMember(i) then
      return i,UnitName(self.PARTYIDS[i])
   end
end

function YakHelpers:eachPartyName()
    return self.partynameiter, self, 0
end

YakHelpers.RAIDIDS = {
   "raid1",
   "raid2",
   "raid3",
   "raid4",
   "raid5",
   "raid6",
   "raid7",
   "raid8",
   "raid9",
   "raid10",
   "raid11",
   "raid12",
   "raid13",
   "raid14",
   "raid15",
   "raid16",
   "raid17",
   "raid18",
   "raid19",
   "raid20",
   "raid21",
   "raid22",
   "raid23",
   "raid24",
   "raid25",
   "raid26",
   "raid27",
   "raid28",
   "raid29",
   "raid30",
   "raid31",
   "raid32",
   "raid33",
   "raid34",
   "raid35",
   "raid36",
   "raid37",
   "raid38",
   "raid39",
   "raid40" }

function YakHelpers:raididiter(i)
   if i >= GetNumRaidMembers() then --MAX_RAID_MEMBERS then
      return nil
   else
      i = i+1
--      if GetRaidRosterInfo(i) == nil then
--	 return self:raididiter(s,i+1)
--      else
      return i, self.RAIDIDS[i]
--      end
   end
end

function YakHelpers:eachRaidID()
    return self.raididiter, self, 0
end

YakHelpers.RAIDPETIDS = {
   "raidpet1",
   "raidpet2",
   "raidpet3",
   "raidpet4",
   "raidpet5",
   "raidpet6",
   "raidpet7",
   "raidpet8",
   "raidpet9",
   "raidpet10",
   "raidpet11",
   "raidpet12",
   "raidpet13",
   "raidpet14",
   "raidpet15",
   "raidpet16",
   "raidpet17",
   "raidpet18",
   "raidpet19",
   "raidpet20",
   "raidpet21",
   "raidpet22",
   "raidpet23",
   "raidpet24",
   "raidpet25",
   "raidpet26",
   "raidpet27",
   "raidpet28",
   "raidpet29",
   "raidpet30",
   "raidpet31",
   "raidpet32",
   "raidpet33",
   "raidpet34",
   "raidpet35",
   "raidpet36",
   "raidpet37",
   "raidpet38",
   "raidpet39",
   "raidpet40" }

function YakHelpers:raidpetiditer(i)
    if i > MAX_RAID_MEMBERS then
        return nil
    else
        i = i+1
        if GetRaidRosterInfo(i) == nil then
	   return self:raidpetiditer(i+1)
        else
	   return i, self.RAIDPETIDS[i]
        end
    end
end

function YakHelpers:eachRaidPetID()
    return self.raidpetiditer, self, 0
end



function YakHelpers:nextTankID(t,k)
    local knew
    local vnew
    knew, vnew = next(t,k)
    if knew== nil or vnew == nil then
        return nil
    else
       return knew, self.RAIDIDS[vnew[1]]
    end
end


function YakHelpers:eachCTRATankID()
    return self.nextTankID, CT_RATarget.MainTanks, nil
end


local function activate( self, oldlib, olddeactivate )
   if oldlib then -- if upgrading
      --nothing
   end

   -- make sure all basic variables are available, n/a in this case
   -- if not self.some_var then self.some_var = 0 end

   if olddeactivate then -- clean up old library
      olddeactivate(oldlib)
   end
end

AceLibrary:Register(YakHelpers, MAJOR_VERSION, MINOR_VERSION, activate )
YakHelpers = nil
