local MAJOR_VERSION = "YakObserver-2.0"
local MINOR_VERSION = 0

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local YakObserver = {}

YakObserver.SUBJECT = {}

function YakObserver.SUBJECT:Create()
   local result = {Observers = {}}
   setmetatable( result, self )
   self.__index = self
   return result
end

function YakObserver.SUBJECT:RegisterObserver(obs)
   -- shouldn't be able to register twice
   local found = false
   for k,v in ipairs(self.Observers) do
      if v == obs then
	 found = true
      end
   end
   if not found then
      table.insert(self.Observers, obs)
   end
end

function YakObserver.SUBJECT:UnregisterObserver(obs)
   --find index
   local index = -1
   for k,v in ipairs(self.Observers) do
      if v == obs then
	 index = k
      end
   end
   if index ~= -1 then
      table.remove( self.Observers, index )
   end
end

function YakObserver.SUBJECT:NotifyObservers(event)
   for k,v in ipairs(self.Observers) do
      v:OnSubjectChange(self, event)
   end
end

YakObserver.OBSERVER = {}

function YakObserver.OBSERVER:Create()
   local result = {}
   setmetatable( result, self )
   self.__index = self
   return result
end

function YakObserver.OBSERVER:OnSubjectChange( subject, event )
end

local function activate( self, oldlib, olddeactivate )
end

AceLibrary:Register(YakObserver, MAJOR_VERSION, MINOR_VERSION, activate )

YakObserver = nil
